/*
 * agn-demo.c
 *
 * Application Test using DiaCanvas2 0.6.0 for Agnubis 
 *
*/


#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <diacanvas/dia-canvas-view.h>
#include <diacanvas/dia-canvas-i18n.h>
#include <diacanvas/dia-canvas-line.h>
#include <diacanvas/dia-canvas-box.h>
#include <diacanvas/dia-canvas-text.h>
#include <diacanvas/dia-canvas-image.h>
#include <diacanvas/dia-placement-tool.h>
#include <libgnomecanvas/gnome-canvas-util.h>
#include <libgnomecanvas/gnome-canvas-line.h>
#include "dia-textbox.h"

#define TEXT_TYPE "text"
#define IMAGE_TYPE "image"


DiaCanvas *canvas = 0;
guint view_counter = 0;
guint text_counter = 0;
guint image_counter = 0;

static void item_destroyed (gpointer data, GObject * obj);
static void display_window_destroy (GtkObject * obj, gpointer data);
static void menu_item_cb (gpointer callback_data, guint callback_action,
			  GtkWidget * widget);
static gboolean key_press_event (GtkWidget * widget, GdkEventKey * key,
				 gpointer data);
GtkWidget *display_canvas_view (GtkWidget * view);
static gint canvas_item_event (DiaCanvasItem * item, DiaEvent * event);


enum
{
	FILE_QUIT,
	FILE_START,
	VIEW_ZOOM_IN,
	VIEW_ZOOM_OUT,
	VIEW_ZOOM_100,
	ELEMENT_TEXT,
	ELEMENT_IMAGE
};



static GtkItemFactoryEntry menu_items[] = {
	{"/_File", NULL, 0, 0, "<Branch>"},
	{"/File/sep1", NULL, NULL, 0, "<Separator>"},
	{"/File/_Start", "<control>S", menu_item_cb, FILE_START},
	{"/File/_Quit", "<control>Q", menu_item_cb, FILE_QUIT},

	{"/_View", NULL, 0, 0, "<Branch>"},
	{"/View/Zoom _In", NULL, menu_item_cb, VIEW_ZOOM_IN},
	{"/View/Zoom _Out", NULL, menu_item_cb, VIEW_ZOOM_OUT},
	{"/View/_Zoom 100%", NULL, menu_item_cb, VIEW_ZOOM_100},

	{"/_Element", NULL, 0, 0, "<Branch>"},
	{"/Element/Add Text", NULL, menu_item_cb, ELEMENT_TEXT},
	{"/Element/Add Image", NULL, menu_item_cb, ELEMENT_IMAGE},

};

static int nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

static void
item_destroyed (gpointer data, GObject * obj)
{
	g_message ("Item destroyed: %s at 0x%p",
		   g_type_name (G_OBJECT_TYPE (obj)), obj);
}


static gboolean
key_press_event (GtkWidget * widget, GdkEventKey * key, gpointer data)
{
	if (key->keyval == GDK_Escape)
	{
		gtk_widget_destroy (widget);
		return TRUE;
	}

	return FALSE;
}

static gint
canvas_item_event (DiaCanvasItem * item, DiaEvent * event)
{
	GdkCursor *cursor;
	GtkWidget *menu = 0, *menu_item = 0;
	switch (event->type)
	{
	case DIA_EVENT_2BUTTON_PRESS:
		g_print ("Double Click \n");
		break;
	case DIA_EVENT_BUTTON_PRESS:
		switch (event->button.button)
		{
		case 1:
			g_print ("Left Button Click \n");
			cursor = gdk_cursor_new (GDK_HAND1);
			/*dia_canvas_item_grab(item);   */
			break;

		case 3:
			g_print ("Right Button Click \n");
			menu = gtk_menu_new ();
			menu_item =
				gtk_menu_item_new_with_label (_
							      ("Properties"));
			gtk_menu_append (GTK_MENU (menu), menu_item);
			gtk_widget_show (menu_item);

			menu_item = gtk_menu_item_new_with_label (_("Copy"));
			gtk_menu_append (GTK_MENU (menu), menu_item);
			gtk_widget_show (menu_item);

			menu_item = gtk_menu_item_new_with_label (_("Paste"));
			gtk_menu_append (GTK_MENU (menu), menu_item);
			gtk_widget_show (menu_item);

			menu_item =
				gtk_menu_item_new_with_label (_("Delete"));
			gtk_menu_append (GTK_MENU (menu), menu_item);
			gtk_widget_show (menu_item);

			gtk_widget_show (menu);
			gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL,
					NULL, event->button.button, 0);
			break;
		default:
			break;
		}
		break;
	default:
		break;
	}
	return FALSE;
}

static void
menu_item_cb (gpointer callback_data, guint callback_action,
	      GtkWidget * widget)
{

	DiaCanvasView *view = callback_data;

	dia_canvas_push_undo (view->canvas, NULL);

	switch (callback_action)
	{
	case FILE_QUIT:
		gtk_main_quit ();
		break;
	case FILE_START:
	{
		GtkWidget *window = 0, *viewzoom = 0;
		gint w, h;

		viewzoom = dia_canvas_view_new (view->canvas, TRUE);
		dia_canvas_view_set_zoom (DIA_CANVAS_VIEW (viewzoom), 2.0);
		w = gdk_screen_width ();
		h = gdk_screen_height ();
		window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

		gnome_canvas_set_scroll_region (GNOME_CANVAS (viewzoom), 0.0,
						0.0, (double) w, (double) h);
		gtk_window_set_decorated (GTK_WINDOW (window), FALSE);
		gtk_window_maximize (GTK_WINDOW (window));

		gtk_window_set_position (GTK_WINDOW (window),
					 GTK_WIN_POS_CENTER_ALWAYS);
		gtk_container_add (GTK_CONTAINER (window), viewzoom);

		gtk_widget_show (viewzoom);
		gtk_widget_show (window);
		gtk_window_resize (GTK_WINDOW (window), w, h);

		gtk_signal_connect (GTK_OBJECT (window), "key_press_event",
				    GTK_SIGNAL_FUNC (key_press_event), NULL);

		break;
	}

	case VIEW_ZOOM_IN:
	{
		GtkAdjustment *adj =
			g_object_get_data (G_OBJECT (view), "zoom");
		gtk_adjustment_set_value (adj, adj->value + 0.1);
		break;
	}
	case VIEW_ZOOM_OUT:
	{
		GtkAdjustment *adj =
			g_object_get_data (G_OBJECT (view), "zoom");
		gtk_adjustment_set_value (adj, adj->value - 0.1);
		break;
	}
	case VIEW_ZOOM_100:
	{
		GtkAdjustment *adj =
			g_object_get_data (G_OBJECT (view), "zoom");
		gtk_adjustment_set_value (adj, 1.0);
		break;
	}
	case ELEMENT_TEXT:
	{
		DiaCanvasItem *item;
		PangoFontDescription *font_desc;
		gchar *text, *id_text, *text_name;

		text_counter++;
		id_text = g_new (gchar, 1);
		g_snprintf (id_text, 2, "%d", text_counter);

		font_desc = pango_font_description_from_string ("sans 30");
		text_name = g_strconcat (TEXT_TYPE, id_text, NULL);
		text = g_strconcat ("Agnubis text label", id_text, NULL);

		item = dia_canvas_item_create (DIA_TYPE_TEXTBOX,
					       "width", 300.0,
					       "height", 100.0, NULL);
		dia_canvas_groupable_add (canvas->root, item);
		g_object_weak_ref (G_OBJECT (item), item_destroyed, NULL);


		/* TODO: This is a hack! into API DiaCanvas 0.6.2 */

		g_object_set (DIA_TEXTBOX (item)->text,
			      "font", font_desc, "text", text, NULL);


		dia_canvas_item_move (item, 10.0, 10.0);
		pango_font_description_free (font_desc);
		g_object_set_data (G_OBJECT (item), "id", id_text);
		g_object_set_data (G_OBJECT (item), "type", TEXT_TYPE);
		g_object_set_data (G_OBJECT (item), text_name, item);
		g_object_set_data (G_OBJECT (canvas), text_name, item);

		g_signal_connect (item, "event",
				  G_CALLBACK (canvas_item_event), NULL);


		break;
	}
	case ELEMENT_IMAGE:
	{
		DiaCanvasItem *item;
		GdkPixbuf *pixbuf;
		GError *error = NULL;
		gchar *image_name, *id_image;

		pixbuf = gdk_pixbuf_new_from_file ("./image.png", &error);
		if (error)
		{
			g_error ("Error occured");
		}

		image_counter++;
		id_image = g_new (gchar, 1);
		g_snprintf (id_image, 2, "%d", image_counter);
		image_name = g_strconcat (IMAGE_TYPE, id_image, NULL);

		item = dia_canvas_item_create (dia_canvas_image_get_type (),
					       "image", pixbuf,
					       "width",
					       (gdouble)
					       gdk_pixbuf_get_width (pixbuf),
					       "height",
					       (gdouble)
					       gdk_pixbuf_get_height (pixbuf),
					       NULL);
		dia_canvas_groupable_add (canvas->root, item);
		g_object_weak_ref (G_OBJECT (item), item_destroyed, NULL);

		dia_canvas_item_move (item, 100.0, 100.0);
		g_object_set_data (G_OBJECT (item), "id", id_image);
		g_object_set_data (G_OBJECT (item), "type", IMAGE_TYPE);
		g_object_set_data (G_OBJECT (item), image_name, item);
		g_object_set_data (G_OBJECT (canvas), image_name, item);

		g_signal_connect (item, "event",
				  G_CALLBACK (canvas_item_event), NULL);

		gdk_pixbuf_unref (pixbuf);
		break;
	}
	default:
		g_assert_not_reached ();
	}
};


static void
zoom_changed (GtkAdjustment * adj, gpointer data)
{
	dia_canvas_view_set_zoom (data, adj->value);
}

static void
display_window_destroy (GtkObject * obj, gpointer data)
{
	view_counter--;
	if (view_counter == 0)
		gtk_main_quit ();
}


GtkWidget *
display_canvas_view (GtkWidget * view)
{
	GtkWidget *window = 0;
	GtkWidget *vbox = 0;
	GtkWidget *hbox = 0;
	GtkWidget *table = 0;
	GtkWidget *w = 0;
	GtkWidget *frame = 0;
	GtkAdjustment *adj = 0;
	GtkAccelGroup *accel_group;
	GtkItemFactory *item_factory;


	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (window),
			      "AgnCanvas Demostration, using DiaCanvas2 0.6.0");
	gtk_window_set_policy (GTK_WINDOW (window), TRUE, TRUE, FALSE);
	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
	gtk_window_set_default_size (GTK_WINDOW (window), 400, 400);
	gtk_signal_connect (GTK_OBJECT (window), "destroy",
			    GTK_SIGNAL_FUNC (display_window_destroy), NULL);
	g_object_set_data (G_OBJECT (view), "window", window);
	vbox = gtk_vbox_new (FALSE, 4);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
	gtk_widget_show (vbox);

	accel_group = gtk_accel_group_new ();
	item_factory =
		gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
				      accel_group);
	g_object_set_data_full (G_OBJECT (window), "<main>", item_factory,
				(GDestroyNotify) g_object_unref);
	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
	gtk_item_factory_create_items (item_factory, nmenu_items, menu_items,
				       view);
	w = gtk_item_factory_get_widget (item_factory, "<main>");
	gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
	gtk_widget_show (w);

	w = gtk_label_new ("Agnubis Canvas text: time for crash it!.");
	gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
	gtk_widget_show (w);
	hbox = gtk_hbox_new (FALSE, 4);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show (hbox);

	/* Zoom */
	w = gtk_label_new ("Zoom:");
	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
	gtk_widget_show (w);

	adj = GTK_ADJUSTMENT (gtk_adjustment_new
			      (1.00, 0.05, 5.00, 0.05, 0.50, 0.50));
	gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
			    (GtkSignalFunc) zoom_changed, view);
	g_object_set_data (G_OBJECT (view), "zoom", adj);

	w = gtk_spin_button_new (adj, 0.0, 2);
	gtk_widget_set_usize (w, 50, 0);
	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
	gtk_widget_show (w);

	/* Layout the stuff */
	table = gtk_table_new (2, 2, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
	gtk_table_set_col_spacings (GTK_TABLE (table), 4);
	gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
	gtk_widget_show (table);

	frame = gtk_frame_new (NULL);
	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
	gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 0, 1,
			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
			  GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0, 0);
	gtk_widget_show (frame);

	gnome_canvas_set_scroll_region (GNOME_CANVAS (view), 0, 0, 600, 450);
	gtk_container_add (GTK_CONTAINER (frame), view);
	GTK_WIDGET_SET_FLAGS (view, GTK_CAN_DEFAULT | GTK_CAN_FOCUS);
	g_object_set_data (G_OBJECT (view), "frame", frame);
	gtk_widget_show (view);

	w = gtk_hscrollbar_new (GTK_LAYOUT (view)->hadjustment);
	gtk_table_attach (GTK_TABLE (table), w, 0, 1, 1, 2,
			  GTK_EXPAND | GTK_FILL | GTK_SHRINK, GTK_FILL, 0, 0);
	gtk_widget_show (w);

	w = gtk_vscrollbar_new (GTK_LAYOUT (view)->vadjustment);
	gtk_table_attach (GTK_TABLE (table), w, 1, 2, 0, 1, GTK_FILL,
			  GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0, 0);
	gtk_widget_show (w);

	gtk_container_add (GTK_CONTAINER (window), vbox);
	gtk_widget_show (view);
	gtk_widget_show (window);

	gtk_widget_grab_focus (view);
	gtk_widget_grab_default (view);

	view_counter++;
	return window;

}

int
main (int argc, char *argv[])
{

	GtkWidget *view = 0, *window = 0;
	DiaRectangle extents = { -300.0, -300.0, 300.0, 300.0 };


#ifdef ENABLE_NLS
	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain (PACKAGE);
#endif

	gtk_init (&argc, &argv);

	gtk_widget_push_colormap (gdk_rgb_get_cmap ());


	canvas = g_object_new (DIA_TYPE_CANVAS,
			       "static_extents", FALSE,
			       "extents", &extents,
			       "grid_bg", DIA_COLOR (200, 250, 250), NULL);
	gtk_widget_pop_colormap ();


	view = dia_canvas_view_new (NULL, TRUE);
	window = display_canvas_view (view);
	g_object_set (view, "canvas", canvas, NULL);
	canvas->allow_undo = TRUE;
	g_object_unref (canvas);

	gtk_main ();

	return 0;
}
