#include "string.h"
#include "agnubis-presentation-control.h"

AgnubisPresentationControl * 
agnubis_presentation_control_new (gchar *filename)
{
	AgnubisPresentationControl *self;
	GtkWidget *window;
	gpointer mainwin;
	DiaCanvasView *view;

	g_print("AgnubisPresentationControl::new() - enter\n");

	// create an instance of AgnubisPresentationControl;
	self = (AgnubisPresentationControl *)g_malloc(sizeof(AgnubisPresentationControl));
	g_print("AgnubisPresentationControl::new() - Address: %i\n", self);

	// create a presentation and set *presentation to it
	self->presentation = agnubis_presentation_new_presentation(self, filename /*"~/unsaved.agnubis"*/ );
	self->modified = FALSE;

	// create the gtkliststore and fill it with items
	self->store = gtk_list_store_new(3,
			G_TYPE_STRING, // Page number string
			G_TYPE_STRING, // Master name
			G_TYPE_STRING, // Page Title
			G_TYPE_INT);   // real page number

	self->windows = NULL;

	agnubis_presentation_presentation_init(self->presentation);
	// not needed, init does it already.
	//agnubis_presentation_control_refresh_slide_store(self);
	
	// open one window
	// TP: Dont open window here. Need to create separately.
	// TP: open_file needs to create presentation control
	// TP: but it does not want a new window.
	agnubis_presentation_control_create_window(self);
	

	// add self to the (global) list of presentation controls
	agnubis_presentation_controls = g_slist_prepend(agnubis_presentation_controls, self);

	mainwin = self->windows->data;
	view = g_object_get_data( G_OBJECT(mainwin), "view");
	agnubis_presentation_change_slide(self->presentation, view, 0);

	g_print("AgnubisPresentationControl::new() - leave\n");

	return self;
}

AgnubisPresentationControl *
agnubis_presentation_control_new_with_window(GtkWidget *window)
{
	// this creates a presentation control having an existing window
}

void 
agnubis_presentation_control_refresh_slide_store (AgnubisPresentationControl * self)
{
	gint i, len;
	GtkTreeIter iter;
	gchar *num_string, *title_string;

	g_print("AgnubisPresentationControl::refreshSlideStore() - enter\n");
	len = agnubis_presentation_num_of_slides(self->presentation);
	g_print("AgnubisPresentationControl::refreshSlideStore() - Slide has %i entries\n", len);

	for (i = 0; i < len; i++)
	{
		gtk_list_store_append (self->store, &iter);
		num_string = g_new0(gchar, 6);
		g_snprintf(num_string, 5, "%i.", (i+1));

		title_string = g_new0(gchar, strlen(agnubis_presentation_get_slide_title (self->presentation, i)));
		title_string = agnubis_presentation_get_slide_title (self->presentation, i);

		gtk_list_store_set (self->store, &iter,
				0, num_string,
				1, "Field 1",
				2, title_string,
				3, i,
				-1);

		g_free(num_string);
		g_free(title_string);
	}

	g_print("AgnubisPresentationControl::refreshSlideStore() - leave\n");
	return;
}

void 
agnubis_presentation_control_update_store_selection (AgnubisPresentationControl * self)
{
	gint i = 0;
	while(g_slist_nth_data(self->windows, i)!=0)
	{
		g_print("%i",i);
	}
}

void 
agnubis_presentation_control_close_window (AgnubisPresentationControl * control, GtkWidget * window)
{
	g_print("AgnubisPresentationControl::closeWindow() - enter\n");
	g_print("AgnubisPresentationControl::closeWindow() - %i windows before\n", g_slist_length(control->windows));

	if (g_slist_length(control->windows) <= 1)
	{
		agnubis_presentation_autosave(control->presentation);
		control->windows = g_slist_remove(control->windows, window);
		agnubis_presentation_control_destroy(control);
	}
	else
	{
		control->windows = g_slist_remove(control->windows, window);

		gtk_widget_destroy(window);

	}
	g_print("AgnubisPresentationControl::closeWindow() - %i windows after\n", g_slist_length(control->windows));
	g_print("AgnubisPresentationControl::closeWindow() - leave\n");
        return;
}



GtkWidget *
agnubis_presentation_control_create_window (AgnubisPresentationControl * control)
{
	GtkWidget	* window;

	g_print("AgnubisPresentationControl::createWindow() - enter\n");
	g_print("AgnubisPresentationControl::createWindow() - %i windows before\n", g_slist_length(control->windows));
	window = agnubis_gui_new(control);

	control->windows = g_slist_prepend(control->windows, window);

	g_print("AgnubisPresentationControl::createWindow() - %i windows after\n", g_slist_length(control->windows));
	g_print("AgnubisPresentationControl::createWindow() - leave\n");
	
	gtk_widget_show(window);
	
        return window;
}

void 
agnubis_presentation_control_destroy (AgnubisPresentationControl * control)
{
        if (g_slist_length(control->windows) >= 1)
	{
		// Close all the windows safely
		g_message("There are still open windows");
		return;
	}

	agnubis_presentation_controls = g_slist_remove(agnubis_presentation_controls, control);

	if (g_slist_length(agnubis_presentation_controls) <= 0)
	{
		// no open presentation controls
		gtk_main_quit();
	}

	return;
}

#line 140 "agnubis-presentation-control.gob"
void 
agnubis_presentation_control_load (AgnubisPresentationControl * self, gchar * filename)
#line 334 "agnubis-presentation-control.c"
{
#line 141 "agnubis-presentation-control.gob"
	
		
		agnubis_presentation_control_refresh_slide_store(self);
		return;
}
#line 349 "agnubis-presentation-control.c"
#undef __GOB_FUNCTION__

#line 147 "agnubis-presentation-control.gob"
gboolean 
agnubis_presentation_control_save (AgnubisPresentationControl * self, gchar * filename)
#line 355 "agnubis-presentation-control.c"
{
#line 149 "agnubis-presentation-control.gob"
	
	        //FIXME: Save *presentation to *filename
		return;
}
#line 369 "agnubis-presentation-control.c"
#undef __GOB_FUNCTION__

#line 154 "agnubis-presentation-control.gob"
GtkListStore * 
agnubis_presentation_control_get_slide_list_store (AgnubisPresentationControl * self)
{
#line 156 "agnubis-presentation-control.gob"
	
		g_print("AgnubisPresentationControl::getSlideListStore() - enter and exit\n");
		return self->store;
		
	}
#line 390 "agnubis-presentation-control.c"
#undef __GOB_FUNCTION__

#line 162 "agnubis-presentation-control.gob"
AgnubisPresentation * 
agnubis_presentation_control_get_presentation (AgnubisPresentationControl * self)
#line 396 "agnubis-presentation-control.c"
{
#line 164 "agnubis-presentation-control.gob"
	
		return self->presentation;
}
#line 409 "agnubis-presentation-control.c"
#undef __GOB_FUNCTION__

#line 168 "agnubis-presentation-control.gob"
GtkListStore * 
agnubis_presentation_control_get_store (AgnubisPresentationControl * self)
#line 415 "agnubis-presentation-control.c"
{
#line 170 "agnubis-presentation-control.gob"
	
		 return self->store;
}
#line 428 "agnubis-presentation-control.c"
#undef __GOB_FUNCTION__

