/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* Agnubis Callback definitions
 * 
 * AUTHORS
 *		Sven Herzberg        <herzi@abi02.de>
 *		Luis Manuel Bentez  <luisma@gnome.org>
 *      Alvaro del Castillo  <acs@barrapunto.com>
 *
 * Copyright (C) 2002 GNOME Foundation
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PAR0TICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "callbacks.h"
#include "interface.h"
#include "preferences.h"
#include "gui-utils/misc.h"
#include <agnubis-object-image.h>
#include <agnubis-object-box.h>
#include <agnubis-slide-design.h>
#include <agnubis-presentation.h>
#include <agnubis-presentation-control.h>
#include <agnubis-presentation-full-screen.h>

void on_main_window_delete_cb(GtkWidget *mainwin, gpointer data)
{
	AgnubisPresentationControl	* control;
	control = g_object_get_data(G_OBJECT(mainwin), "control");

	agnubis_presentation_control_close_window(control, mainwin);

	return;
}

// menu: file

static gboolean
file_is_dir (const gchar *file)
{
	struct stat sb;

	if ((stat (file, &sb) == 0) && S_ISDIR (sb.st_mode)) {
		return TRUE;
	}
	
	return FALSE;
}

void on_file_new(BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	AgnubisPresentationControl *control;
	GtkWidget *window;
	
	window = GTK_WIDGET(mainwin);

	control = g_object_get_data(
			G_OBJECT(window),
			"control");

	newfile_dialog_new(control->modified);
	return;
}

void on_file_open(BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	GtkWidget			* file_sel,
					* checkbutton;
	gint				  response;
	const gchar			* filename;
	AgnubisPresentationControl	* control;
	AgnubisPresentation		* presentation;
	AgnubisSlideDesign		* design;
	AgnubisCanvas			* canvas;
	DiaCanvasView			* view;
	
	file_sel = gtk_file_selection_new (_("Open a file"));
	checkbutton = gtk_check_button_new_with_label(_("Open in a new window"));

	gtk_widget_show(checkbutton);
/*	gtk_box_pack_start(
			GTK_BOX(GTK_DIALOG(GTK_FILE_SELECTION(file_sel)->fileop_dialog)->vbox),
			checkbutton,
			FALSE,
			FALSE,
			0);
*/
	gtk_window_set_modal (GTK_WINDOW (file_sel), TRUE);

	gtk_widget_show (file_sel);

	response = gtk_dialog_run (GTK_DIALOG (file_sel));

	if (response == GTK_RESPONSE_OK) {
		filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_sel));
		if (file_is_dir (filename)) {
			filename = NULL;
		}
	}
	if (!filename) return;

	view = g_object_get_data( G_OBJECT(mainwin), "view");
	control = g_object_get_data (G_OBJECT(mainwin), "control");	
	
	presentation = agnubis_presentation_control_get_presentation(control);
	g_print("%p,%p,%p\n", view, control, presentation);
	agnubis_presentation_view_disconnect(presentation, view);

	// Create new control and start using its mainwin
	control = agnubis_presentation_control_new((gchar*)filename);
	mainwin = control->windows->data;

	// Ask also the 'new window' checkbox
	if(g_slist_length(control->windows) >= 2)
	{
		// (01) unlink the window from the control
		// (02) create a new control having this window
	}

	presentation = agnubis_presentation_control_get_presentation(control);
	view = g_object_get_data( G_OBJECT(mainwin), "view");
	g_object_set_data(G_OBJECT(mainwin), "control", control);

	g_print("%p,%p,%p\n", view, control, presentation);

	agnubis_presentation_first_slide(presentation, view);

	gtk_widget_destroy (file_sel);

	return;
}

void on_close_window (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	AgnubisPresentationControl	* control;
	GtkWidget			* window;

	window = GTK_WIDGET(mainwin);

	control = g_object_get_data(G_OBJECT(window), "control");

	agnubis_presentation_control_close_window(control, window);
	
	return;
}

void on_file_exit(BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	quit_dialog_show(NULL,NULL);

	return;
}

// menu: edit

void on_edit_preferences(
		BonoboUIComponent *uic,
		gpointer mainwin,
		const gchar *verbname)
{
	preferences_dialog_show();
}

// menu: insert

void on_insert_image (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	GtkWidget        *file_sel;
	gint              response;
	const gchar      *imagefile = NULL;
	GtkWidget *dialog;
	DiaCanvasView *view;
	AgnubisObjectImage *image;
	
	file_sel = gtk_file_selection_new (_("Load the image from file"));
	gtk_window_set_modal (GTK_WINDOW (file_sel), TRUE);
	gtk_widget_show (file_sel);
	response = gtk_dialog_run (GTK_DIALOG (file_sel));

	if (response == GTK_RESPONSE_OK) {
		imagefile = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_sel));
	}
	
	gtk_widget_destroy (file_sel);

	if (imagefile != NULL) {
		g_message (_("We are going to open the image file: %s"), imagefile);
		
		view = g_object_get_data( G_OBJECT(mainwin), "view");
	
		image = agnubis_object_image_new ();

		// Show the image
		g_object_set (image, "file_image", imagefile, NULL);
		
		// Wait for user click
		agnubis_object_draw_on_click ((AgnubisObject *)image, view);
	}
	return;
}

void on_insert_label (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	/* FIXME : AgnubisObjectText don't work 
	AgnubisObjectText *text;
	AgnubisCanvas *canvas;
	GtkWidget *view;
	
	view = g_object_get_data( G_OBJECT(mainwin), "view");
	canvas = (AgnubisCanvas *) dia_canvas_view_get_canvas (DIA_CANVAS_VIEW(view));
	
	text = agnubis_object_text_new ();		
	
	agnubis_object_draw ((AgnubisObject *)text, canvas);
	*/

	return;
}

void on_insert_text (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	return;
}

void 
on_insert_box (BonoboUIComponent *uic, 
	       gpointer mainwin, 
	       const gchar *verbname)
{
	AgnubisObjectBox *box;
	GtkWidget *view;

	g_message ("Inserting a box ...");
	
	view = g_object_get_data( G_OBJECT(mainwin), "view");
	
	box = agnubis_object_box_new ();		
	
	agnubis_object_draw_on_click ((AgnubisObject *) box, (DiaCanvasView*)view);

	return;
}

void 
on_insert_line (BonoboUIComponent *uic, 
		gpointer mainwin, 
		const gchar *verbname)
{
	/*
	AgnubisObjectLine *line;
	GtkWidget *view;

	g_message ("Inserting a line ...");
	
	view = g_object_get_data( G_OBJECT(mainwin), "view");
	
	line = agnubis_object_line_new ();
	
	agnubis_object_draw ((AgnubisObject *) line, view);
	*/

	return;
}

void 
on_insert_zigzag_line (BonoboUIComponent *uic, 
		gpointer mainwin, 
		const gchar *verbname)
{
	/*
	AgnubisObjectLine *zigzag;
	GtkWidget *view;

	g_message ("Inserting a zigzag line ...");
	
	view = g_object_get_data( G_OBJECT(mainwin), "view");
	canvas = (AgnubisCanvas *) dia_canvas_view_get_canvas (DIA_CANVAS_VIEW(view));
	
	zigzag = agnubis_object_line_new ();
	
	agnubis_object_draw ((AgnubisObject *) zigzag, view);
	*/

	return;
}

void 
on_insert_polyline (BonoboUIComponent *uic, 
		gpointer mainwin, 
		const gchar *verbname)
{
	/*
	AgnubisObjectLine *polyline;
	GtkWidget *view;

	g_message ("Inserting a polyline ...");
	
	view = g_object_get_data( G_OBJECT(mainwin), "view");
	
	polyline = agnubis_object_line_new ();
	
	agnubis_object_draw ((AgnubisObject *) polyline, view);
	*/

	return;
}

void 
on_insert_arrow (BonoboUIComponent *uic, 
		gpointer mainwin, 
		const gchar *verbname)
{
	/*
	AgnubisObjectLine *arrow;
	GtkWidget *view;

	g_message ("Inserting an arrow...");
	
	view = g_object_get_data( G_OBJECT(mainwin), "view");
	
	arrow = agnubis_object_arrow_new ();
	
	g_object_set (arrow,
		      "has_head", TRUE,
		      "head_a", 20.0,
		      "head_b", 20.0,
		      "head_c", 20.0,
		      "head_d", 20.00,
		      NULL);
	
	agnubis_object_draw ((AgnubisObject *) arrow, view);
	*/

	return;
}

// menu: windows

void on_create_window (
		BonoboUIComponent	* uic,
		gpointer		  mainwin,
		const gchar		* verbname)
{
	AgnubisPresentationControl	* control;
	GtkWidget			* window;
	AgnubisPresentation             * presentation;
	DiaCanvasView                   * view;
	gint                              slidenum;

	window = GTK_WIDGET(mainwin);

	control = g_object_get_data(
			G_OBJECT(window),
			"control");

	presentation = agnubis_presentation_control_get_presentation(control);
	view = g_object_get_data( G_OBJECT(mainwin), "view");
	slidenum = agnubis_presentation_get_current_page(presentation, view);

	agnubis_presentation_control_create_window(control);

	mainwin = control->windows->data;

	presentation = agnubis_presentation_control_get_presentation(control);
	view = g_object_get_data( G_OBJECT(mainwin), "view");
	agnubis_presentation_change_slide(presentation, view, slidenum);
	

	return;
}

// menu: help

void on_help_info (
		BonoboUIComponent *uic,
		gpointer mainwin, const gchar *verbname)
{
	agnubis_about();
}

//
// Slide Browser
//

void
on_slide_browser_selection_changed_cb(
		GtkTreeSelection *selection,
		gpointer mainwin)
{
	AgnubisPresentationControl *control;
	AgnubisPresentation *presentation;
	GtkListStore *store;
	GtkTreeIter iter;
	GtkTreePath *path;
	GtkWidget *view=0;
	gint slide_num;
	
	if(!selection)
	        return;
	g_print("type=%s\n", G_OBJECT_TYPE_NAME(G_OBJECT(mainwin)));
	control = g_object_get_data(G_OBJECT(mainwin), "control");
	
	store = control->store;

	gtk_tree_selection_get_selected (selection, NULL, &iter);
	path = gtk_tree_model_get_path (GTK_TREE_MODEL(store), &iter);
	slide_num = gtk_tree_path_get_indices(path) [0];

	presentation = agnubis_presentation_control_get_presentation(control);
	//presentation = control->presentation;
	
        view = g_object_get_data(G_OBJECT(mainwin),"view");

	agnubis_presentation_change_slide(presentation, DIA_CANVAS_VIEW(view), slide_num);

	return;
}

/* Main Toolbar */



//gboolean on_button_zoom_in_cb (GtkWidget* widget,  gpointer mainwin)
void on_view_zoom_in (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	//Zoom in
       
	GtkWidget *view=0;
        GtkAdjustment *adj =0;

	g_message("ZOOM IN\n");

        view = g_object_get_data(G_OBJECT(mainwin),"view");
        adj  = g_object_get_data (G_OBJECT(view), "zoom");
        gtk_adjustment_set_value (adj, adj->value + 0.1);
	
}



//gboolean on_button_zoom_out_cb (GtkWidget* widget, gpointer mainwin)
void on_view_zoom_out (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	// Zoom out

	GtkWidget *view=0;
        GtkAdjustment *adj =0;

        view = g_object_get_data(G_OBJECT(mainwin),"view");
        adj  = g_object_get_data (G_OBJECT(view), "zoom");
	g_message("ZOOM OUT %f\n", adj->value);
        gtk_adjustment_set_value (adj, adj->value - 0.1);
	
}

void on_view_zoom_fit_cb (GtkAdjustment *adj2, gpointer mainwin)
{
	GtkWidget *view=0;
        GtkAdjustment *adj =0;
	gint width, height;
	gint origwidth = 1024;
	gint origheight = 768;
	gfloat xzoom, yzoom;
	GtkAdjustment *horiz, *vert;
	gboolean toggle;
        view = (GtkWidget*)g_object_get_data(G_OBJECT(mainwin),"view");
        adj  = g_object_get_data (G_OBJECT(view), "zoom");
	
	horiz = gtk_layout_get_hadjustment(GTK_LAYOUT(view));
	vert = gtk_layout_get_vadjustment(GTK_LAYOUT(view));
	width = horiz->page_size;
	height = vert->page_size;
		
	xzoom = ((gfloat)width)/(gfloat)origwidth;
	yzoom = ((gfloat)height)/(gfloat)origheight;
	if (xzoom < yzoom) yzoom = xzoom;
	else xzoom = yzoom;

	xzoom = 1.0/xzoom;

	toggle = (gboolean)g_object_get_data(G_OBJECT(mainwin), "zoom_fit");
	if (toggle)
		gtk_adjustment_set_value (adj, xzoom);
}

//gboolean on_button_zoom_fit_cb (GtkWidget* widget, gpointer mainwin)
void on_view_zoom_fit (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)

{
	gboolean toggle;
	GtkWidget *view=0;
	GtkWidget *hscrollbar, *vscrollbar;
	toggle = (gboolean)g_object_get_data(G_OBJECT(mainwin), "zoom_fit");
	if (toggle) toggle = FALSE;
	else toggle = TRUE;
	g_object_set_data(G_OBJECT(mainwin), "zoom_fit", (gpointer)toggle);
	on_view_zoom_fit_cb(NULL, mainwin);
        view = (GtkWidget*)g_object_get_data(G_OBJECT(mainwin),"view");

	hscrollbar = g_object_get_data(G_OBJECT(view), "hscroll");
	vscrollbar = g_object_get_data(G_OBJECT(view), "vscroll");

	if (toggle) 
	{
	gtk_widget_hide(hscrollbar);
	gtk_widget_hide(vscrollbar);
	}
	else
	{
	gtk_widget_show(hscrollbar);
	gtk_widget_show(vscrollbar);
	}


	//FIXME: Add functionality
	// This function is called, when the user wants to open an existing
	// presentation.
	// This function should identify the window, it was activated in
	// and then call a function in backend.c, that makes this zoom to
	// make the slide fit into the canvas_view and creates an undo-entry
	// in the undo-list.
}




void on_view_zoom_100 (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	// Zoom 100%
						
	GtkWidget *view=0;
        GtkAdjustment *adj =0;
	g_message("ZOOM 100%\n");


        view = g_object_get_data(G_OBJECT(mainwin),"view");
        adj  = g_object_get_data (G_OBJECT(view), "zoom");
        gtk_adjustment_set_value (adj, 1.0);
	
}

void on_zoom_changed_cb (GtkWidget *adj, gpointer view)
{
	// Zoom adjust to user selection
	
	//dia_canvas_view_set_zoom (view, GTK_ADJUSTMENT(adj)->value);
	gnome_canvas_set_pixels_per_unit(view, 1.0/GTK_ADJUSTMENT(adj)->value);

	return;
}

void on_zoom_selector_change_cb (GtkWidget *zoom_selector, GtkWidget *mainwin)
{
	// Selector to adjust the Zoom

	GtkWidget *view=0;
        GtkAdjustment *adj =0;
	gfloat increment;

        view = g_object_get_data(G_OBJECT(mainwin),"view");
        adj  = g_object_get_data (G_OBJECT(view), "zoom");
        
	increment = atoi(gtk_entry_get_text(GTK_ENTRY(zoom_selector)));
	//gtk_adjustment_set_value (adj, (increment/100));

	return;
}

/* Format Toolbar */
void on_change_font_in_selection_cb (GtkWidget *font_selector, GtkWidget *mainwin)
{
	//FIXME: Add functionality
	// This function is called, when the user wants change the font
	// in a selected passage.
	// This function should indetify the window it was called from
	// and then call a function in backend.c, that makes the selection's
	// font change and that creates an entry in the undo-list.
}

void on_change_font_size_in_selection_cb (GtkWidget *size_selector, GtkWidget *mainwin)
{
	//FIXME: Add functionality
}




/* Presentation Toolbar */

void on_go_previous (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	DiaCanvasView                   * view = NULL;
	AgnubisPresentationControl	* control;
	AgnubisPresentation             * presentation = NULL;

	view = g_object_get_data( G_OBJECT(mainwin), "view");
	control = g_object_get_data(G_OBJECT(mainwin), "control");
	presentation = agnubis_presentation_control_get_presentation(control);
	if (presentation)
		agnubis_presentation_prev_slide(presentation, view);

	return;
}

void on_go_next (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	DiaCanvasView *view = NULL;
	AgnubisPresentationControl	* control;
	AgnubisPresentation *presentation = NULL;

	view = g_object_get_data( G_OBJECT(mainwin), "view");
	control = g_object_get_data(G_OBJECT(mainwin), "control");

	presentation = agnubis_presentation_control_get_presentation(control);

	if (presentation)
		agnubis_presentation_next_slide(presentation, view);

	return;
}

void on_go_first (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	DiaCanvasView *view = NULL;
	AgnubisPresentationControl	* control;
	AgnubisPresentation *presentation = NULL;

	view = g_object_get_data( G_OBJECT(mainwin), "view");
	control = g_object_get_data(G_OBJECT(mainwin), "control");
	presentation = agnubis_presentation_control_get_presentation(control);

	if (presentation)
		agnubis_presentation_first_slide(presentation, view);
	return;

}

void on_go_last (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	DiaCanvasView *view = NULL;
	AgnubisPresentationControl	* control;
	AgnubisPresentation *presentation = NULL;

	view = g_object_get_data( G_OBJECT(mainwin), "view");
	control = g_object_get_data(G_OBJECT(mainwin), "control");
	presentation = agnubis_presentation_control_get_presentation(control);

	if (presentation)
		agnubis_presentation_last_slide(presentation, view);
	return;
}

void on_presentation_slide_new (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	DiaCanvasView *view = NULL;
	AgnubisPresentationControl	* control;
	AgnubisPresentation *presentation = NULL;

	view = g_object_get_data( G_OBJECT(mainwin), "view");
	control = g_object_get_data(G_OBJECT(mainwin), "control");
	presentation = agnubis_presentation_control_get_presentation(control);

	agnubis_presentation_implement_change(presentation, "insert_slide");
	agnubis_presentation_refresh_slide(presentation, view);

	// should remove the following after "changes" work.
	agnubis_presentation_save_as(presentation, "data/tmp.agnubis");
}

void on_presentation_slide_delete (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	DiaCanvasView *view = NULL;
	AgnubisPresentationControl	* control;
	AgnubisPresentation *presentation = NULL;

	view = g_object_get_data( G_OBJECT(mainwin), "view");
	control = g_object_get_data(G_OBJECT(mainwin), "control");
	presentation = agnubis_presentation_control_get_presentation(control);

	agnubis_presentation_implement_change(presentation, "delete_slide");
	agnubis_presentation_refresh_slide(presentation, view);
}


gboolean on_button_add_slide_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
{


	//FIXME: Add functionality
	// This function is called, when the user wants to add a slide
	// to the presentation.
	// This function should identify the window, it was activated in
	// and then call a function in backend.c, that opens a dialog to
	// add a slide (should contain where to add the slide and what
	// kind of slide to add - as there can be 'embty', 'from template,...').
	return TRUE;
}

gboolean on_button_remove_slide_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
{
	//FIXME: Add functionality
	// This function is called, when the user wants to remove a slide
	// of the presentation.
	// This function should identify the window, it was activated in
	// and then call a function in backend.c, that opens a dialog to
	// select the slide to be removed (remember the confirmation:
	// 'do you really want to delete...').


	return TRUE;
}

gboolean on_button_start_presentation_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
{


	//FIXME: Add functionality
	// This function is called, when the user wants to preview /
	// start the presentation.
	// This function should identify the window, it was activated in
	// and then call a function in backend.c, that starts the
	// presentation.
	return TRUE;
 }

void on_start_presentation (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname)
{
	AgnubisPresentationControl *control;
	GtkWidget *window;
	AgnubisPresentationFullScreen *fullscreen;
	
	window = GTK_WIDGET(mainwin);

	control = g_object_get_data(
			G_OBJECT(window),
			"control");

	fullscreen = agnubis_presentation_full_screen_new_fullscreen(control);
	gnome_dialog_set_parent(GNOME_DIALOG(fullscreen), GTK_WINDOW(mainwin));
	gnome_dialog_run(GNOME_DIALOG(fullscreen));
}






