/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* Agnubis Callback declarations
 *
 * Authors:
 *      Sven Herzberg        <herzi@abi02.de>
 *      Alvaro del Castillo  <acs@barrapunto.com>
 *
 * Copyright (C) 2002 GNOME Foundation
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef __CALLBACKS_H__
#define __CALLBACKS_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gnome.h>
#include <libbonoboui.h>
 
#ifdef __cplusplus
extern "C" {
#endif
	/* Main Window */
	void on_main_window_delete_cb (GtkWidget *mainwin, gpointer data);

	// menu: file
	void on_file_new(BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_file_open(BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_close_window (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_file_exit(BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	
	// menu: view
	void on_start_presentation (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);

	// menu: edit
	void on_edit_preferences(BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);

	// menu: insert
	void on_insert_image(BonoboUIComponent *uic, gpointer mainwin, const gchar* verbname);
	void on_insert_label(BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_insert_text (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_insert_box  (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_insert_line (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_insert_zigzag_line (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_insert_polyline (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_insert_arrow (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);

	// menu: go
	void on_go_previous (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_go_next (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_go_first (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_go_last (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);

	// menu: windows
	void on_create_window (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);

	// slide browser
	void on_slide_browser_selection_changed_cb(GtkTreeSelection *selection, gpointer window);
	
	// menu: help
	void on_help_info(BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);

	/* Main Toolbar */
	gboolean on_button_undo_cb      (GtkWidget* widget, GdkEvent *event, gpointer data);
	gboolean on_button_redo_cb      (GtkWidget* widget, GdkEvent *event, gpointer data);
	void on_zoom_changed_cb         (GtkWidget *adj, gpointer view);
	void on_zoom_selector_change_cb (GtkWidget *zoom_selector, GtkWidget *mainwin);
	void on_view_zoom_in (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_view_zoom_out (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_view_zoom_100 (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_view_zoom_fit (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_view_zoom_fit_cb (GtkAdjustment *adj2, gpointer mainwin);


	/* Text Toolbar */
	void on_change_font_in_selection_cb      (GtkWidget *font_selector, GtkWidget *mainwin);
	void on_change_font_size_in_selection_cb (GtkWidget *size_selector, GtkWidget *mainwin);

	/* Presentation Toolbar */
	gboolean on_button_add_slide_cb          (GtkWidget* widget, GdkEvent *event, gpointer data);
	gboolean on_button_remove_slide_cb       (GtkWidget* widget, GdkEvent *event, gpointer data);
	gboolean on_button_start_presentation_cb (GtkWidget* widget, GdkEvent *event, gpointer data);

	void on_presentation_slide_new (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);
	void on_presentation_slide_delete (BonoboUIComponent *uic, gpointer mainwin, const gchar *verbname);


#ifdef _cplusplus
}
#endif

#endif /* __CALLBACKS_H__ */
