/*  RhythmBox
 *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
 *                     Marco Pesenti Gritti <marco@it.gnome.org>
 *                     Bastien Nocera <hadess@hadess.net>
 *                     Seth Nickell <snickell@stanford.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id: misc.c,v 1.4 2002/06/26 22:07:09 herzi Exp $
 *	Added some functions by apg & Luisma
 */

#include <libgnome/gnome-util.h>
#include <libgnome/gnome-i18n.h>
#include <gtk/gtk.h>
#include <string.h>
#include <unistd.h>
#include <glade/glade.h>

#include "misc.h"

#if 0
static void error_dialog_destroy (GtkObject *dialog, gpointer data);
#endif

/**
 * user_file: returns the pathname of mp shared files (e.g., player.glade)
 *
 * fname: just the filename with no path information
 * critical: critical file? (halt if not found)
 *
 * galeon CREDITS
 */
char *
user_file (char *fname, gboolean critical)
{
	static GHashTable *already_found = NULL;
	gchar *alternative[6];
	gchar *file;
	gint i;
	
	/* create cache hash table if it doesnt already exist */
	if (already_found == NULL)
	{
		already_found = g_hash_table_new (g_str_hash, g_str_equal);
	}

        /* Have we already found this? */
	file = g_hash_table_lookup (already_found, fname);
	if (file != NULL) 
	{
		return g_strdup (file);
	}

	/* try the default */
	file = g_strconcat (g_get_home_dir (), "/.agnubis/", fname, NULL);
	
	/* success? */
	if (g_file_exists (file))
	{
		return file;
	}
	g_free(file);

	/* specify alternate locations in order of precedence */
	alternative[0] = g_strdup (fname);
	alternative[1] = g_strconcat (AGNUBIS_DATA_DIR "pixmaps/", fname, NULL);
	alternative[2] = g_strconcat ("data/", fname, NULL);
	alternative[3] = g_strconcat ("../data/", fname, NULL);
	//alternative[4] = g_strconcat (SHARE_DIR "/", fname, NULL);
	alternative[4] = NULL;  /* NULL terminator needed */
	
	/* select one of the alternatives */
	file = NULL;
	for (i = 0; alternative[i] != NULL; i++)
	{
		if (file == NULL && g_file_exists (alternative[i])) 
		{
			file = alternative[i];
                        /* don't warn for the install default */
			if (i != 4)
			{
				g_message ("Using %s (usually OK)\n", file);
			}
		}
		else
		{
			/* free unused string */
			g_free (alternative[i]);
		}
	}

	/* check for success */
	if (file != NULL)
	{
		/* add it to the set of found files */
		g_hash_table_insert (already_found, g_strdup (fname), 
				     g_strdup (file));
	}
	/* if nothing then theres an error */
	else if (critical)
	{
		g_error (_("%s not found"), fname);
	}

	/* return result */
	return file;
}

/**
 * error_dialog: popups an error dialog with the message @msg and exits
 * (like g_error() does)
 */
void
error_dialog (gchar *msg)
{
	GtkWidget *dialog;

	dialog = gtk_message_dialog_new (NULL,
			0,
			GTK_MESSAGE_ERROR,
			GTK_BUTTONS_OK,
			msg);

	gtk_dialog_run (GTK_DIALOG(dialog));
	gtk_widget_destroy (dialog);

	exit (1);
}

#if 0
/*
 * response_check: checks the answer to a error_dialog_with_details
 */
static void
error_dialog_destroy (GtkObject *dialog, gpointer data)
{
	exit(0);
}
#endif

/**
 * error_dialog_with_details: popups an error dialog with a disclosure button
 * and exits
 */
void
error_dialog_with_details (gchar *msg, gchar *detailed_msg)
{
#if 0
	GtkDialog *dialog;

	dialog = eel_show_info_dialog_with_details
		(msg, _("Rhythmbox error"), detailed_msg, NULL);

	g_signal_connect (G_OBJECT(dialog), "destroy",
			G_CALLBACK(error_dialog_destroy), NULL);

	gtk_main();
#endif
}


/**
 * string_request_dialog: popups a dialog asking for a string, with @default as
 * the default string.
 * @parent: parent window
 * @msg: message
 * @title: title of the dialog
 * @def_string: default answer
 * returns: 
 */
char *
string_request_dialog (GtkWindow *parent, char *msg, char *title,
		char *def_string)
{
	GtkWidget *dialog, *label, *entry, *vbox;
	gint response;
	char *string;

	dialog = gtk_dialog_new_with_buttons (title,
			parent,
			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
			GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, NULL);

	/* add a vbox in the dialog vbox */
	vbox = gtk_vbox_new(FALSE, 8);
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
			vbox);

	/* add the label */
	label = gtk_label_new (msg);
	gtk_container_add (GTK_CONTAINER (vbox), label);

	/* add the entry */
	entry = gtk_entry_new ();
	if (def_string != NULL)
		gtk_entry_set_text (GTK_ENTRY (entry), def_string);
	gtk_container_add (GTK_CONTAINER (vbox), entry);

	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
			GTK_RESPONSE_OK);

	gtk_widget_show_all (dialog);
	response = gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_hide (dialog);
	if (response == GTK_RESPONSE_REJECT) {
		gtk_widget_destroy (dialog);
		return NULL;
	}

	string = gtk_editable_get_chars (GTK_EDITABLE (entry),
			0, -1);
	gtk_widget_destroy (dialog);
	return string;
}

/* Destroy the quit_dialog */
void quit_dialog_destroy (GtkWidget *button, gpointer dialog_pointer)
{
	GtkWidget *dialog = GTK_WIDGET (dialog_pointer);
	
	gtk_widget_destroy (dialog);
	
	return;
}

/* Destroy the close_dialog */
void close_dialog_destroy (GtkWidget *button, gpointer dialog_pointer)
{
	GtkWidget *dialog = GTK_WIDGET (dialog_pointer);
	
	gtk_widget_destroy (dialog);
	
	return;
}

/* Function to show the quit_dialog... Do you want to exit? */
void quit_dialog_show(GtkWidget *button, gpointer dialog_pointer)
{
	GladeXML *xml_data=0;
	GtkWidget *quit_dialog, *button_quit, *button_cancel;

	xml_data = glade_xml_new (AGNUBIS_GLADE_DIR "/quit.glade", "quit_dialog", NULL);

	quit_dialog = glade_xml_get_widget (xml_data, "quit_dialog");

	gtk_window_set_modal (GTK_WINDOW (quit_dialog), TRUE);
	
	button_quit = glade_xml_get_widget (xml_data, "button_quit");
	g_signal_connect (
			G_OBJECT (button_quit),
			"clicked",
			G_CALLBACK (gtk_main_quit),
			quit_dialog);

	button_cancel = glade_xml_get_widget (xml_data, "button_cancel");
	g_signal_connect (
			G_OBJECT (button_cancel),
			"clicked",
			G_CALLBACK 
			(quit_dialog_destroy),
			quit_dialog);
	
	return;
}

/* Function to show the close_dialog... Do you want to close? */
void close_dialog_show(GtkWidget *button, gpointer dialog_pointer)
{
	GladeXML *xml_data=0;
	GtkWidget *close_dialog, *button_close, *button_cancel;

	xml_data = glade_xml_new (AGNUBIS_GLADE_DIR "/quit.glade", "close_dialog", NULL);

	close_dialog = glade_xml_get_widget (xml_data, "close_dialog");

	gtk_window_set_modal (GTK_WINDOW (close_dialog), TRUE);
	
	button_close = glade_xml_get_widget (xml_data, "button_close_close");
	g_signal_connect (
			G_OBJECT (button_close),
			"clicked",
			G_CALLBACK (gtk_main_quit),
			close_dialog);

	button_cancel = glade_xml_get_widget (xml_data, "button_close_cancel");
	g_signal_connect (
			G_OBJECT (button_cancel),
			"clicked",
			G_CALLBACK 
			(close_dialog_destroy),
			close_dialog);
	
	return;
}
