/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* Agnubis GUI description
 * 
 * AUTHORS
 *       lvaro Pea Gonzlez <alvaropg@mac.com>
 *       Jorge Bustos Bustos  <jorge@gnome.cl>
 *       Luis Manuel Benitez  <luisma@gnome.org>
 *       Sven Herzberg        <herzi@abi02.de>
 *
 * Copyright (C) 2002 GNOME Foundation
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <gal/widgets/gtk-combo-stack.h>
#include <gal/widgets/gtk-combo-text.h>
#include <gal/e-paned/e-hpaned.h>

//#include <gtk/gtk.h>
#include <gnome.h>
#include <glade/glade.h>
#include <libbonoboui.h>

#include "agnubis-presentation-control.h"
#include "backend.h"
#include "callbacks.h"
#include "interface.h"
#include "main.h"

#define EVENT_METHOD(i, x) GTK_WIDGET_GET_CLASS(i)->x

GtkWidget* create_client_area(GtkWidget *mainwin);
BonoboDockItem* create_toolbar_main(GtkWidget *mainwin);
BonoboDockItem* create_toolbar_text(GtkWidget *mainwin);
BonoboDockItem* create_toolbar_slide(GtkWidget *mainwin);
GtkWidget* create_toolbar_pres(GtkWidget *mainwin);

/* BonoboUIVerbList */
static const BonoboUIVerb ui_verbs[] = {
	/* menu: file */
	BONOBO_UI_VERB ("FileNew", on_file_new),
	BONOBO_UI_VERB ("FileOpen", on_file_open),
//	BONOBO_UI_VERB ("FileImport", on_file_import),
//	BONOBO_UI_VERB ("FileSave", on_file_save),
//	BONOBO_UI_VERB ("FileSaveAs", on_file_save_as),
//	BONOBO_UI_VERB ("FileSaveCopy", on_file_save_copy),
//	BONOBO_UI_VERB ("FileRevertToSaved", on_file_revert_to_saved),
//	BONOBO_UI_VERB ("FileExportHTML", on_file_export_html),
//	BONOBO_UI_VERB ("PrintPageSetup", on_print_page_setup),
//	BONOBO_UI_VERB ("PrintPreview", on_print_preview);
//	BONOBO_UI_VERB ("Print", on_file_print),
//	BONOBO_UI_VERB ("SendFileTo", on_file_send_to),
//	BONOBO_UI_VERB ("FileProperties", on_file_properties),
	BONOBO_UI_VERB ("CloseWindow", on_close_window),
	BONOBO_UI_VERB ("ExitAgnubis", on_file_exit),

	/* menu: edit */
//	BONOBO_UI_VERB ("EditUndo", on_edit_undo),
//	BONOBO_UI_VERB ("EditRedo", on_edit_redo),
//	BONOBO_UI_VERB ("EditCut", on_edit_cut),
//	BONOBO_UI_VERB ("EditCopy", on_edit_copy),
//	BONOBO_UI_VERB ("EditPaste", on_edit_paste),
//	BONOBO_UI_VERB ("EditPasteSpecial", on_edit_paste_special),
//	BONOBO_UI_VERB ("EditDelete", on_edit_delete),
//	BONOBO_UI_VERB ("EditSelectAll", on_edit_select_all),
//	BONOBO_UI_VERB ("EditDeselectAll", on_edit_deselect_all),
//	BONOBO_UI_VERB ("EditFind", on_edit_find),
//	BONOBO_UI_VERB ("EditFindNext", on_edit_find_next),
//	BONOBO_UI_VERB ("EditFindPrevious", on_edit_find_previous),
//	BONOBO_UI_VERB ("EditReplace", on_edit_replace),
	BONOBO_UI_VERB ("EditPreferences", on_edit_preferences),

	/* menu: view */
//	BONOBO_UI_VERB ("ViewNormal", on_view_normal),
//	BONOBO_UI_VERB ("ViewPreHTML", on_view_preview_html),
//	BONOBO_UI_VERB ("ViewPreSlide", on_preview_current_slide),
//	BONOBO_UI_VERB ("ViewGrids", on_view_grids),
//	BONOBO_UI_VERB ("ViewRulers", on_view_rulers),
//	BONOBO_UI_VERB ("ViewNotes", on_view_notes),
//	BONOBO_UI_VERB ("ViewToolbars", on_view_toolbars),
	BONOBO_UI_VERB ("ViewZoomIn", on_view_zoom_in),
	BONOBO_UI_VERB ("ViewZoomOut", on_view_zoom_out),
	BONOBO_UI_VERB ("ViewZoom100", on_view_zoom_100),
	BONOBO_UI_VERB ("ViewZoomFit", on_view_zoom_fit),
//	BONOBO_UI_VERB ("ViewRefresh", on_view_refresh),
//	BONOBO_UI_VERB ("PreviewPage", on_preview_page),
	BONOBO_UI_VERB ("PresentationStart", on_start_presentation),

	/* menu:insert */
//	BONOBO_UI_VERB ("InsertSlideNumber", on_insert_slide_number),
//	BONOBO_UI_VERB ("InsertTime", on_time_insert),
//	BONOBO_UI_VERB ("InsertSymbol", on_insert_symbol),
//	BONOBO_UI_VERB ("InsertNote", on_insert_note),
//	BONOBO_UI_VERB ("InsertText", on_insert_text),
	BONOBO_UI_VERB ("InsertLabel", on_insert_label),
//	BONOBO_UI_VERB ("InsertLink", on_insert_link),
	BONOBO_UI_VERB ("InsertLine", on_insert_line),
	BONOBO_UI_VERB ("InsertArrow", on_insert_arrow),
	BONOBO_UI_VERB ("InsertZigZagLine", on_insert_zigzag_line),
	BONOBO_UI_VERB ("InsertPolyline", on_insert_polyline),
	BONOBO_UI_VERB ("InsertRect", on_insert_box),
//	BONOBO_UI_VERB ("InsertEllipse", on_insert_ellipse),
	BONOBO_UI_VERB ("InsertImage", on_insert_image),
//	BONOBO_UI_VERB ("InserGraph", on_insert_graph),
//	BONOBO_UI_VERB ("InsertTable", on_insert_table),
//	BONOBO_UI_VERB ("InsertSound", on_insert_sound),
//	BONOBO_UI_VERB ("InsertVideo", on_insert_video),
//	BONOBO_UI_VERB ("InsertObject", on_insert_object),
//	BONOBO_UI_VERB ("InsertFile", on_insert_file),

	/* menu: format */
//	BONOBO_UI_VERB ("FormatFont", on_format_font),
//	BONOBO_UI_VERB ("FormatBull", on_format_bull),
//	BONOBO_UI_VERB ("AlignLeft", on_align_left),
//	BONOBO_UI_VERB ("AlignCenter", on_align_center),
//	BONOBO_UI_VERB ("AlignRight", on_align_right),
//	BONOBO_UI_VERB ("AlignFill", on_align_fill),
//	BONOBO_UI_VERB ("FormatPosLeft", on_format_left),
//	BONOBO_UI_VERB ("FormatPosCenter", on_format_center),
//	BONOBO_UI_VERB ("FormatPosRight", on_format_right),
//	BONOBO_UI_VERB ("FormatPosTop", on_format_top),
//	BONOBO_UI_VERB ("FormatPosMiddle", on_format_middle),
//	BONOBO_UI_VERB ("FormatPosBottom", on_format_bottom),
//	BONOBO_UI_VERB ("FormatDesign", on_format_design),
//	BONOBO_UI_VERB ("FormatBackground", on_format_background),

	// menu: go
	BONOBO_UI_VERB ("GoPrevious", on_go_previous),
	BONOBO_UI_VERB ("GoNext", on_go_next),
//	BONOBO_UI_VERB ("GoJumpTo", on_go_jump_to),
	BONOBO_UI_VERB ("GoFirst", on_go_first),
	BONOBO_UI_VERB ("GoLast", on_go_last),

	// menu: tools
//	BONOBO_UI_VERB ("ToolSpellCheck", on_tool_spell_check),
//	BONOBO_UI_VERB ("ToolPlugIn", on_tool_plugin),

	// menu: windows
	BONOBO_UI_VERB ("CreateWindow", on_create_window),
//	BONOBO_UI_VERB ("WindowSaveAll", on_window_save_all),
//	BONOBO_UI_VERB ("WindowCloseAll", on_window_close_all),

	// menu: help
	BONOBO_UI_VERB ("HelpInfo", on_help_info),

	// toolbar: main
//	BONOBO_UI_VERB ("HelpContent", on_help_content),

	// toolbar: text
//	BONOBO_UI_VERB ("FontBold", on_font_bold),
//	BONOBO_UI_VERB ("FontItalic", on_font_italic),
//	BONOBO_UI_VERB ("FontUnderline", on_font_underline),
//	BONOBO_UI_VERB ("FontStriketrough", on_font_strikethrough),
//	BONOBO_UI_VERB ("ListBulleted", on_list_bulleted),
//	BONOBO_UI_VERB ("ListNumbered", on_list_numbered),

	BONOBO_UI_VERB ("PresentationSlideNew", on_presentation_slide_new),
	BONOBO_UI_VERB ("PresentationSlideDelete", on_presentation_slide_delete),
//	BONOBO_UI_VERB ("StyleNext", on_style_next),
//	BONOBO_UI_VERB ("StylePrev", on_style_prev),
	

	BONOBO_UI_VERB_END
};

/* this injects a widget as a bonoboui-control
 * the code was copied from gnumeric 1.1.3 (gui-util.c)
 */
void inject_widget_into_bonoboui (BonoboUIComponent *uic, GtkWidget *widget, char const *path)
{
	BonoboControl *control;

	gtk_widget_show_all(widget);
	control = bonobo_control_new(widget);
	bonobo_ui_component_object_set(
			uic,
			path,
			BONOBO_OBJREF(control),
			NULL);
}

void agnubis_about(void)
{
	GladeXML *about_xml=0;
	GtkWidget *agnubis_about;
	GError *error=NULL;
	GdkPixbuf *splash;

	/* Load file XML */
	about_xml = glade_xml_new (AGNUBIS_DATA_DIR "/agnubis/glade/agnubis-about.glade","agnubis_about", NULL);
	agnubis_about = glade_xml_get_widget (about_xml, "agnubis_about");

	/* Load Pixbuf from file */
	splash = gdk_pixbuf_new_from_file(AGNUBIS_PIXMAPS_DIR "/i-web.png",&error);   
	if (error) 
		g_error (_("Error occured"));
  
	/* g_object_set_data(G_OBJECT(agnubis_about),"logo",splash); */

	gtk_widget_show(agnubis_about);

	return;
}

void close_main_window(GtkWidget *mainwin)
{
	agnubis_app_list = g_slist_remove (agnubis_app_list, mainwin);
	gtk_widget_hide (GTK_WIDGET(mainwin));

	/* And if there aren't any other windows */
	if (g_slist_length(agnubis_app_list) == 0)
	{
		g_message(_("Exiting..."));
		gtk_main_quit ();
	}

	return;
}


gint
agnubis_main_window_key_press_event(GtkWidget *d, GdkEventKey *e, GtkWidget *mainwin)
{
	GtkWindow *window = GTK_WINDOW(d);
	AgnubisPresentation *presentation = NULL;
	DiaCanvasView *view = NULL;
	AgnubisPresentationControl *control = NULL;
	control = g_object_get_data(G_OBJECT(mainwin), "control");
	view = g_object_get_data(G_OBJECT(mainwin), "view");
	presentation = agnubis_presentation_control_get_presentation(control);
	switch(e->keyval)
		{
		case GDK_Page_Up:
			gtk_widget_event(window->focus_widget, (GdkEvent*)e);
			agnubis_presentation_prev_slide(presentation, view);
			return TRUE;
		case GDK_Page_Down:
			gtk_widget_event(window->focus_widget, (GdkEvent*)e);
			agnubis_presentation_next_slide(presentation, view);
			return TRUE;
		case GDK_Home:
			gtk_widget_event(window->focus_widget, (GdkEvent*)e);
			agnubis_presentation_first_slide(presentation, view);
			return TRUE;
		case GDK_End:
			gtk_widget_event(window->focus_widget, (GdkEvent*)e);
			agnubis_presentation_last_slide(presentation, view);
			return TRUE;
		case GDK_BackSpace:
		case GDK_Left:
		case GDK_Up:
		case GDK_Right:
		case GDK_Down:
		case GDK_Return:
		case GDK_KP_Space:
		case GDK_space:
		case GDK_Escape:			
			gtk_widget_event(window->focus_widget, (GdkEvent*)e);
			return TRUE;
		};
	return FALSE;
}


GtkWidget*
agnubis_gui_new (AgnubisPresentationControl *control)
{
	/* Creating Main Window */
	GtkWidget *mainwin, *widget;
	BonoboUIComponent *ui_component;
	GList *l, *icons;
	gint i, index=0, len=0;

	g_print("AgnubisGui::new() - enter\n");

	//FIXME: use a cleaner way to add the icon list
	widget = gtk_image_new_from_file(AGNUBIS_DATA_DIR "/pixmaps/agnubis.png");
	icons = NULL;

	icons = g_list_prepend(icons, gtk_image_get_pixbuf(GTK_IMAGE(widget)));

	//gtk_window_set_default_icon_list(icons);

	mainwin = bonobo_window_new (GETTEXT_PACKAGE, "Agnubis");

	gtk_signal_connect(GTK_OBJECT(mainwin), "key_press_event", GTK_SIGNAL_FUNC(agnubis_main_window_key_press_event), mainwin);

	g_object_set_data(G_OBJECT(mainwin),
			"control",
			control);

	bonobo_ui_engine_config_set_path (
			bonobo_window_get_ui_engine(BONOBO_WINDOW(mainwin)),
			"/agnubis/UIConfig/kvps");

	ui_component = bonobo_ui_component_new_default ();

	bonobo_ui_component_set_container (
			ui_component,
			BONOBO_OBJREF(bonobo_window_get_ui_container(BONOBO_WINDOW(mainwin))),
			NULL);

	bonobo_ui_util_set_ui (
			ui_component,
			AGNUBIS_DATA_DIR "/gnome-2.0/ui/",
			"GNOME_Agnubis.xml",
			"agnubis",
			NULL);

	agnubis_app_list = g_slist_prepend (
			agnubis_app_list,
			mainwin);
	g_signal_connect (mainwin, "delete_event", G_CALLBACK(on_main_window_delete_cb), mainwin);

	/* filling main window's content area */
	bonobo_window_set_contents (
			BONOBO_WINDOW(mainwin),
			create_client_area(mainwin));

	/* linking ui with signal callbacks */
	bonobo_ui_component_add_verb_list_with_data (
			ui_component,
			ui_verbs,
			mainwin);

// creating bonoboui's controls
	// undo-selector
	widget = gtk_combo_stack_new("gtk-undo", TRUE);
	inject_widget_into_bonoboui(
			ui_component,
			widget,
			"/ToolbarMain/EditUndo");

	// redo-selector
	widget = gtk_combo_stack_new("gtk-redo", TRUE);
	inject_widget_into_bonoboui(
			ui_component,
			widget,
			"/ToolbarMain/EditRedo");

	// zoom selector
	widget = gtk_combo_text_new(TRUE);
	g_signal_connect(G_OBJECT(GTK_COMBO_TEXT(widget)->entry),
			"activate",
			G_CALLBACK(on_zoom_selector_change_cb),
			mainwin);
	gtk_container_set_border_width(GTK_CONTAINER(widget), 0);
	gtk_combo_box_set_title(GTK_COMBO_BOX(widget), "Zoom");
	gtk_combo_text_add_item(GTK_COMBO_TEXT(widget), "200%", "200%");
	gtk_combo_text_add_item(GTK_COMBO_TEXT(widget), "175%", "175%");
	gtk_combo_text_add_item(GTK_COMBO_TEXT(widget), "150%", "150%");
	gtk_combo_text_add_item(GTK_COMBO_TEXT(widget), "125%", "125%");
	gtk_combo_text_add_item(GTK_COMBO_TEXT(widget), "100%", "100%");
	gtk_combo_text_add_item(GTK_COMBO_TEXT(widget), "75%", "75%");
	gtk_combo_text_add_item(GTK_COMBO_TEXT(widget), "50%", "50%");
	gtk_combo_text_add_item(GTK_COMBO_TEXT(widget), "25%", "25%");
	gtk_combo_text_select_item(GTK_COMBO_TEXT(widget), 6);
	gtk_widget_set_usize(GTK_COMBO_TEXT(widget)->entry, 40, 0);
	inject_widget_into_bonoboui(
			ui_component,
			widget,
			"/ToolbarMain/ZoomSelect");

	// font selector
	widget = gtk_combo_text_new(TRUE);
	g_signal_connect(G_OBJECT(GTK_COMBO_TEXT(widget)->entry),
			"activate",
			G_CALLBACK(on_change_font_in_selection_cb),
			mainwin);
	gtk_combo_box_set_title(GTK_COMBO_BOX(widget), "Font");
	gtk_container_set_border_width(GTK_CONTAINER(widget), 0);
	for (l = font_family_list; l; l = l->next)
	{
		if (l->data) // Avoid empty fonts in the list
		{
			gint tmp = gdk_string_measure(gtk_style_get_font(GTK_COMBO_TEXT(widget)->entry->style), l->data);
			if (tmp > len)
				len = tmp;
			gtk_combo_text_add_item(GTK_COMBO_TEXT(widget), l->data, l->data);
			if (g_strcasecmp("times new roman", l->data) == 0)
				gtk_combo_text_select_item (GTK_COMBO_TEXT(widget), i);
		}
		i++;
	}
	gtk_widget_set_usize(GTK_COMBO_TEXT(widget)->entry, len, 0);
	inject_widget_into_bonoboui(
			ui_component,
			widget,
			"/ToolbarText/FontSelector");

	// font size selector
	widget = gtk_combo_text_new(TRUE);
	g_signal_connect(G_OBJECT(GTK_COMBO_TEXT(widget)->entry),
			"activate",
			G_CALLBACK(on_change_font_size_in_selection_cb),
			mainwin);
	gtk_combo_box_set_title(GTK_COMBO_BOX(widget), "Size");
	for(i=0; font_sizes[i] != 0; i++)	
	{
		gchar buffer[12];
		g_snprintf(buffer, sizeof(buffer),
				"%d",
				font_sizes[i]);
		gtk_combo_text_add_item(GTK_COMBO_TEXT(widget), buffer, buffer);
	}
	gtk_widget_set_usize(
			GTK_COMBO_TEXT(widget)->entry,
			gdk_string_measure(gtk_style_get_font (GTK_COMBO_TEXT(widget)->entry->style), "888") + 10,
			0);
	gtk_combo_text_select_item(GTK_COMBO_TEXT(widget),5);
	inject_widget_into_bonoboui(
			ui_component,
			widget,
			"/ToolbarText/SizeSelector");

	g_print("AgnubisGui::new() - exit\n");

	return mainwin;
}

BonoboDockItem* create_toolbar_text(GtkWidget* mainwin)
{
	GtkWidget *dockitem, *toolbar, *element;
	GList *l;
	gint len, i, index;

	/* Font Size Selector */
	element = gtk_combo_text_new(TRUE);
	g_signal_connect(G_OBJECT(GTK_COMBO_TEXT(element)->entry),
			"activate",
			G_CALLBACK(on_change_font_size_in_selection_cb),
			mainwin);
	gtk_combo_box_set_title(GTK_COMBO_BOX(element), "Size");
	for(i=0; font_sizes[i] != 0; i++)
	{
		gchar buffer[12];
		g_snprintf(buffer, sizeof(buffer),
				"%d",
				font_sizes[i]);
		gtk_combo_text_add_item(GTK_COMBO_TEXT(element), buffer, buffer);
	}

/* gtk_widget_set_usize(
		GTK_COMBO_TEXT(element)->entry,
		gdk_string_measure(gtk_style_get_font (GTK_COMBO_TEXT(element)->entry->style), "888"),
		0);
*/

	gtk_combo_text_select_item(GTK_COMBO_TEXT(element),5);
	gtk_widget_set_usize(GTK_COMBO_TEXT(element)->entry, 30, 0);
	gtk_widget_show(element);
	gtk_toolbar_append_widget(
			GTK_TOOLBAR(toolbar),
			element,
			"Size",
			"");

	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

	// background color selector
	
	// text color selector

	gtk_widget_show(toolbar);
	gtk_container_add(GTK_CONTAINER(dockitem), toolbar);
	gtk_widget_show(dockitem);
	return BONOBO_DOCK_ITEM(dockitem);
}

GtkWidget* create_client_area(GtkWidget *mainwin)
{
	GtkWidget *notebook=0, *child=0, *label=0;
	GtkWidget *paned=0, *scrolledwindow=0;
	GtkWidget *view=0, *table=0, *hruler=0, *vruler=0, *hscrollbar=0, *vscrollbar=0;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkTreeSelection *selection;
	GtkAdjustment *adj=0;
	AgnubisCanvas *canvas=0;
	AgnubisPresentationControl *control;
	GtkAdjustment *horiz, *vert;

	g_print("AgnubisGui::createClientArea() - enter\n");

	paned = e_hpaned_new();
	e_paned_set_position (E_PANED (paned), 150);
	gtk_widget_show(paned);

	/* Navigation inside of the presentation */
	notebook = gtk_notebook_new();
	gtk_widget_show(notebook);

	control = g_object_get_data(
			G_OBJECT(mainwin),
			"control");

	label = gtk_tree_view_new_with_model(GTK_TREE_MODEL(
				agnubis_presentation_control_get_slide_list_store(control)));

	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW(label), TRUE);

	// column for slide numbers
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(
			_("#"),
			renderer,
			"text",
			0, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(label),
			column);

	// column for master icons
	// FIXME: add this
	
	// column for slide names
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(
			_("Title"),
			renderer,
			"text",
			2, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(label),
			column);

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(label));

	g_signal_connect(G_OBJECT(selection),
			"changed",
			G_CALLBACK(on_slide_browser_selection_changed_cb),
			mainwin);

	gtk_widget_show(label);

	child = gtk_scrolled_window_new(gtk_tree_view_get_hadjustment(GTK_TREE_VIEW(label)),
			gtk_tree_view_get_vadjustment(GTK_TREE_VIEW(label)));

	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (child),
			GTK_SHADOW_ETCHED_IN);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (child),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	gtk_container_add(GTK_CONTAINER(child), label);
	gtk_widget_show(child);
	
	label = gtk_label_new(_("Title"));
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child, label);

	child = gtk_tree_view_new();
	gtk_widget_show(child);
	label = gtk_label_new(_("Preview"));
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child, label);

	e_paned_add1(E_PANED(paned), notebook);

	/* Editing the presentation */
	
	view = dia_canvas_view_new (NULL, FALSE);
        gtk_widget_show(view);

	gtk_widget_set_events (view, GDK_POINTER_MOTION_MASK |
                                 GDK_POINTER_MOTION_HINT_MASK);

	hruler = gtk_hruler_new();
	gtk_widget_show(hruler);
	gtk_ruler_set_metric(GTK_RULER(hruler), GTK_PIXELS);
	gtk_ruler_set_range(GTK_RULER(hruler), -500, 499, 0, 1000);
	g_signal_connect_swapped (G_OBJECT (view), "motion_notify_event",
                              G_CALLBACK (EVENT_METHOD (hruler, motion_notify_event)),
                              hruler);

	vruler = gtk_vruler_new();
	gtk_widget_show(vruler);
	gtk_ruler_set_metric(GTK_RULER(vruler), GTK_PIXELS);
	gtk_ruler_set_range(GTK_RULER(vruler), -500, 499, 0, 1000);
	g_signal_connect_swapped (G_OBJECT (view), "motion_notify_event",
                              G_CALLBACK (EVENT_METHOD (vruler, motion_notify_event)),
                              vruler);

	gnome_canvas_set_scroll_region(GNOME_CANVAS(view), 0, 0, 1024, 768);

	hscrollbar = gtk_hscrollbar_new(GTK_LAYOUT(view)->hadjustment);
	gtk_widget_show(hscrollbar);
	vscrollbar = gtk_vscrollbar_new(GTK_LAYOUT(view)->vadjustment);
	gtk_widget_show(vscrollbar);

	table = gtk_table_new(3, 3, FALSE);
	e_paned_add2(E_PANED(paned), table);
	gtk_widget_show(table);
	gtk_table_attach(GTK_TABLE(table), hruler, 1, 2, 0, 1, GTK_EXPAND|GTK_FILL|GTK_SHRINK, GTK_FILL, 0, 0);
	gtk_table_attach(GTK_TABLE(table), vruler, 0, 1, 1, 2, GTK_FILL, GTK_EXPAND|GTK_FILL|GTK_SHRINK, 0, 0);
	gtk_table_attach(GTK_TABLE(table), view, 1, 2, 1, 2, GTK_EXPAND|GTK_FILL|GTK_SHRINK, GTK_EXPAND|GTK_FILL|GTK_SHRINK, 0, 0);
	gtk_table_attach(GTK_TABLE(table), hscrollbar, 1, 2, 2, 3, GTK_EXPAND|GTK_FILL|GTK_SHRINK, GTK_FILL, 0, 0);
	gtk_table_attach(GTK_TABLE(table), vscrollbar, 2, 3, 1, 2, GTK_FILL, GTK_EXPAND|GTK_FILL|GTK_SHRINK, 0, 0);

	g_object_set_data(G_OBJECT(view), "hscroll", hscrollbar);
	g_object_set_data(G_OBJECT(view), "vscroll", vscrollbar);

	/* Association view-mainwin */
	g_object_set_data(G_OBJECT(mainwin),"view",G_OBJECT(view));
	
	adj = GTK_ADJUSTMENT(gtk_adjustment_new(1.00, 1.0, 5.00, 0.05, 0.50, 0.50));
	g_signal_connect(GTK_OBJECT(adj), "value-changed", G_CALLBACK(on_zoom_changed_cb), view);
	g_object_set_data(G_OBJECT(view), "zoom", adj);



	horiz = gtk_layout_get_hadjustment(GTK_LAYOUT(view));
	vert = gtk_layout_get_vadjustment(GTK_LAYOUT(view));
	g_signal_connect(GTK_OBJECT(horiz), "changed", GTK_SIGNAL_FUNC(on_view_zoom_fit_cb), mainwin);
	g_signal_connect(GTK_OBJECT(vert), "changed", GTK_SIGNAL_FUNC(on_view_zoom_fit_cb), mainwin);
	g_print("AgnubisGui::createClientArea() - leave\n");

	return paned;
}





