/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 * Take from MrProject code
 */
 
/*
 * Copyright (C) 2002 CodeFactory AB
 * Copyright (C) 2002 Richard Hult <rhult@codefactory.se>
 * Copyright (C) 2002 Mikael Hallendal <micke@codefactory.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <config.h>
#include <string.h>
#include <glib.h>
#include <libgnome/gnome-i18n.h>
#include "agnubis-object.h"
#include "agnubis-object-loader.h"

static AgnubisObject *
agnol_load (const gchar *file)
{
	AgnubisObject *object;

	object = g_object_new (AGNUBIS_TYPE_OBJECT, NULL);
	
	object->handle = g_module_open (file, G_MODULE_BIND_LAZY);
	
	if (object->handle == NULL) {
		g_warning (_("Could not open object file '%s'\n"),
			   g_module_error ());
		
		return NULL;
	}

	return object;
}

static GList *
agnol_load_dir (const gchar *path)
{
	GDir*			dir;
	const gchar		*name;
	AgnubisObject	*object;
	GList			*list = NULL;

	dir = g_dir_open (path, 0, NULL);
	if (dir == NULL) {		
		return NULL;
	}
	
	while ((name = g_dir_read_name (dir)) != NULL) {
		if (strncmp (name + strlen (name) - 3, ".so", 3) == 0) {
			gchar *object_name = g_strconcat (path, name, NULL);
			object = agnol_load (object_name);
			if (object) {
				list = g_list_append (list, object);
			}
			
			g_free (object);
		}
	}

	g_dir_close (dir);

	return list;
}

GList *
agnubis_object_loader_load (void)
{
	return agnol_load_dir (AGNUBIS_LIB_DIR "/agnubis/objects/");
}
