/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* Agnubis Core App
 *
 * Authors:
 *      lvaro Pea Gonzlez <alvaropg@mac.com>
 *      Sven Herzberg        <herzi@abi02.de>
 *      Luis Manuel Benitez  <luisma@gnome.org>
 *
 * Copyright (C) 2002 GNOME Foundation
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <glade/glade.h>
#include <gconf/gconf-client.h>

#include "e-splash.h"
#include "newfile.h"
#include "agnubis-presentation-control.h"

GList *font_family_list = NULL;
GList *font_sizes_list = NULL;
static PangoFontFamily **pango_families;
static GStringChunk *size_names;

void font_init(void);

const int font_sizes[] = {
	4, 8, 9, 10, 11, 12, 14, 16, 18,
	20, 22, 24, 26, 28, 36, 48, 72, 0
};


int
main (int argc, char *argv[])
{
	GdkPixbuf *splash_fonts, *splash_prefs, *splash_ui;
	GtkWidget *agnubis_gui, *splash;
	gchar *filename;
	AgnubisPresentationControl *control;
	GConfClient *client;

#ifdef ENABLE_NLS
	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
	textdomain (PACKAGE);
#endif

	gnome_program_init (
			PACKAGE,
			VERSION,
			LIBGNOMEUI_MODULE,
			argc,
			argv,
			GNOME_PARAM_APP_DATADIR,
			NULL,
			NULL);

	glade_gnome_init();

	// init splash screen
	splash_ui = gdk_pixbuf_new_from_file(AGNUBIS_DATA_DIR "/pixmaps/agnubis/splash-ui.png", NULL);
	splash_fonts = gdk_pixbuf_new_from_file(AGNUBIS_DATA_DIR "/pixmaps/agnubis/splash-fonts.png", NULL);
	splash_prefs = gdk_pixbuf_new_from_file(AGNUBIS_DATA_DIR "/pixmaps/agnubis/splash-prefs.png", NULL);

	splash = e_splash_new();
	g_signal_connect(
			G_OBJECT(splash),
			"delete-event",
			G_CALLBACK(gtk_widget_hide_on_delete),
			0);
	gtk_widget_show(splash);
	e_splash_add_icon(E_SPLASH(splash), splash_fonts);
	g_object_unref(G_OBJECT(splash_fonts));
	e_splash_add_icon(E_SPLASH(splash), splash_prefs);
	g_object_unref(G_OBJECT(splash_prefs));
	e_splash_add_icon(E_SPLASH(splash), splash_ui);
	g_object_unref(G_OBJECT(splash_ui));

	// give the splash screen the chance to show itself
	while(gtk_events_pending())
		gtk_main_iteration();

	// font initialization
	e_splash_set_icon_highlight(E_SPLASH(splash), 0, TRUE);
	while(gtk_events_pending())
		gtk_main_iteration();

	font_init();

	// loading preferences
	e_splash_set_icon_highlight(E_SPLASH(splash), 1, TRUE);
	while(gtk_events_pending())
		gtk_main_iteration();

	client = gconf_client_get_default();

	// creating an empty presentation and showing one window
	e_splash_set_icon_highlight(E_SPLASH(splash), 2, TRUE);
	while(gtk_events_pending())
		gtk_main_iteration();
	
	filename = "~/unsaved.agnubis";
	if (argc > 1) {
		filename=argv[1];      
	}

	control = agnubis_presentation_control_new(filename);

	// hiding the splash screen (destroying gives errors)
	gtk_widget_hide(splash);

	gtk_main ();
	return 0;
}

static int
compare_family_pointers_by_name(gconstpointer a, gconstpointer b)
{
	PangoFontFamily * const * const fa = a;
	PangoFontFamily * const * const fb = b;

	return g_utf8_collate(
			pango_font_family_get_name (*fa),
			pango_font_family_get_name (*fb));
}

void font_init(void)
{
	char buffer[6];
	int n_families, i;
	PangoContext *context = gdk_pango_context_get();

	size_names = g_string_chunk_new(128);

	pango_context_list_families(context, &pango_families, &n_families);
	qsort(pango_families, n_families, sizeof(*pango_families), compare_family_pointers_by_name);

	for(i=0; i < n_families; i++)
		font_family_list = g_list_prepend(
				font_family_list,
				(gpointer) pango_font_family_get_name (pango_families[i]));

	font_family_list = g_list_reverse(font_family_list);

	for(i=0; font_sizes[i] != 0; i++)
	{
		sprintf(buffer, "%d", font_sizes[i]);
		font_sizes_list = g_list_prepend(
				font_sizes_list,
				g_string_chunk_insert(size_names, buffer));
	}

	g_object_unref(G_OBJECT(context));
}
