/*
 *  welcome.c :  the agnubis welcome screen
 *
 *  Authors:
 *      Sven Herzberg <herzi@abi02.de>
 *		Luis Manuel Bentez <luisma@gnome.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "interface.h"
#include "newfile.h"
#include "gui-utils/misc.h"

void on_newfile_dialog_close(GtkWidget *button, gpointer dialog_pointer)
{
	GtkWidget	* mainwin, * druid_box, * template_box, *empty_box;
	GtkWidget	* dialog;
	gboolean	* new_control;

	dialog = GTK_WIDGET(dialog_pointer);
	new_control = g_object_get_data(G_OBJECT(dialog), "new_control");

	druid_box = GTK_WIDGET(g_object_get_data(
				G_OBJECT(dialog),
				"druidbox"));
	template_box = GTK_WIDGET(g_object_get_data(
				G_OBJECT(dialog),
				"templatebox"));

	empty_box = GTK_WIDGET(g_object_get_data(
				G_OBJECT(dialog),
				"emptybox"));

	if (TRUE == gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(druid_box)))
	{
		// start the druid
		gtk_widget_destroy(dialog);
		if(*new_control == TRUE)
			g_message("Now I should start the druid for a new window");
		else
			g_message("Now I should start the druid for this window");
		return;
	}
	else if (TRUE == gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(template_box)))
	{
		// start the template selector
		gtk_widget_destroy(dialog);
		if(*new_control == TRUE)
			g_message("Now I should start the template selector for a new window");
		else
			g_message("Now I should start the template selector for this window");
		return;
	}
	else if (TRUE == gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(empty_box)))
	{
		// create an empty presentation
		gtk_widget_destroy(dialog);
		if(*new_control == TRUE)
			g_message("Now I should create an empty presentation for a new window");
		else
			g_message("Now I should create an empty presentation for this window");
		return;
	}
	else
		g_assert_not_reached();

	return;
}


GtkWidget* newfile_dialog_new(gboolean new_window)
{
	GladeXML *xml_data=0;
	GtkWidget *dialog, *button;
	
	xml_data = glade_xml_new (AGNUBIS_DATA_DIR "/agnubis/glade/newfile.glade", "newfile_dialog", NULL);
	dialog = glade_xml_get_widget(xml_data, "newfile_dialog");

	// do things with the 'next' button
	button = glade_xml_get_widget(xml_data, "button_next");
	g_signal_connect (
			G_OBJECT (button),
			"clicked",
			G_CALLBACK (on_newfile_dialog_close),
			dialog);

	// do things with the 'cancel' button
	button = glade_xml_get_widget(xml_data, "button_cancel");
	g_signal_connect (
			G_OBJECT (button),
			"clicked",
			G_CALLBACK (close_dialog_destroy),
			dialog);

	// add data to the dialog
	button = glade_xml_get_widget(xml_data, "radiobox_druid");
	g_object_set_data(
			G_OBJECT(dialog),
			"druidbox",
			button);

	button = glade_xml_get_widget(xml_data, "radiobox_template");
	g_object_set_data(
			G_OBJECT(dialog),
			"templatebox",
			button);

	button = glade_xml_get_widget(xml_data, "radiobox_empty");
	g_object_set_data(
			G_OBJECT(dialog),
			"emptybox",
			button);

	g_object_set_data(
			G_OBJECT(dialog),
			"new_control",
			&new_window);

	return dialog;
}
