/* Agnubis preferences dialog and global options
 * 
 * Authors:
 * 	Sven Herzberg		<herzi@abi02.de>
 * 	Luis Manuel Benitez 	<luisma@gnome.org>
 *
 * Copyright (C) 2002 GNOME Foundation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public Licence as
 * published by the Free Software Foundation; either version 2 of the
 * Licence, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <gtk/gtk.h>
#include <glade/glade.h>
#include "preferences.h"

typedef struct _TreeItem TreeItem;
struct _TreeItem
{
	gchar		* title_tree,
			* title_page,
			* pixmap_tree,
			* pixmap_page,
			* widget_name;
	TreeItem	* child;
};

static TreeItem ui_items[] = {
	{"Toolbars", "Toolbars", AGNUBIS_DATA_DIR "/pixmaps/agnubis/toolbars-small.png", AGNUBIS_DATA_DIR "/pixmaps/agnubis/toolbars.png", "toolbar_page", NULL},
	{NULL}
};

TreeItem global_items[] = {
	{"User interface", "User interface", AGNUBIS_DATA_DIR "/pixmaps/agnubis/stock-icon-index.png", AGNUBIS_DATA_DIR "/pixmaps/agnubis/missing_image.png", "ui_page", ui_items},
	{"Files", "Files", AGNUBIS_DATA_DIR "/pixmaps/agnubis/missing_image.png", AGNUBIS_DATA_DIR "/pixmaps/agnubis/missing_image.png", "files_page", NULL},
	{"Key bindings", "Key bindings", AGNUBIS_DATA_DIR "/pixmaps/agnubis/keyboard-shortcut.png", AGNUBIS_DATA_DIR "/pixmaps/agnubis/keyboard-repeat.png", "keys_page", NULL},
	{NULL}
};

void tree_select_callback(
		GtkTreeSelection	* selection,
		GtkNotebook		* notebook)
{
	GtkTreeModel	* model;
	GtkTreeIter	  iter;
	GtkWidget	* label,
			* image;
	GValue		  value = {0,};

	if(! gtk_tree_selection_get_selected(selection, &model, &iter))
		return;

	label = g_object_get_data (
			G_OBJECT(notebook),
			"label");
	image = g_object_get_data (
			G_OBJECT(notebook),
			"image");

	gtk_tree_model_get_value(model, &iter, 3, &value);
	gtk_label_set_markup(GTK_LABEL(label), g_strconcat("<span size=\"x-large\">", g_value_get_string(&value), "</span>", NULL));
	g_value_unset(&value);

	gtk_tree_model_get_value(model, &iter, 4, &value);
	gtk_image_set_from_icon_set(
			GTK_IMAGE(image),
			gtk_icon_set_new_from_pixbuf(g_value_get_object(&value)),
			GTK_ICON_SIZE_DIALOG);
	g_value_unset(&value);

	gtk_tree_model_get_value(model, &iter, 2, &value);
	gtk_notebook_set_current_page(
			GTK_NOTEBOOK(notebook),
			g_value_get_int(&value));
	g_value_unset(&value);

	return;
}

GtkWidget* preferences_dialog_new();

void preferences_dialog_show()
{
	GtkWidget *dialog;

	//FIXME: only open an new dialog if there is no prefs-dialog yet,
	// else simply raise the existing dialog
	dialog = preferences_dialog_new();
        
	gtk_widget_show_all(dialog);
}

void on_preferences_close(GtkWidget *button, gpointer dialog)
{
	GtkWidget *dialog_widget = GTK_WIDGET(dialog);

	gtk_widget_destroy(dialog_widget);
}

GtkWidget* preferences_dialog_new()
{
	GtkTreeStore		* store,
				* store2;
	GtkTreeViewColumn	* column,
				* column2,
				* column3;
	GtkCellRenderer		* cell,
				* celda;
	GladeXML		* xml;
	GtkWidget		* dialog,
				* notebook,
				* widget,
				* widget2;
	TreeItem		* item = global_items;

	xml = glade_xml_new(AGNUBIS_DATA_DIR "/agnubis/glade/preferences.glade", "dialog", NULL);
	dialog = glade_xml_get_widget(xml, "dialog");
	notebook = glade_xml_get_widget(xml, "notebook");

	g_object_set_data(
			G_OBJECT(notebook),
			"label",
			glade_xml_get_widget(xml, "title_label"));
	g_object_set_data(
			G_OBJECT(notebook),
			"image",
			glade_xml_get_widget(xml, "title_image"));

	// setting up the treeview
	widget = glade_xml_get_widget(xml, "treeview_categories");
	g_object_set_data(G_OBJECT(dialog), "treeview", widget);

	store = gtk_tree_store_new(5,
			GDK_TYPE_PIXBUF,	// small icon for the tree
			G_TYPE_STRING,		// label for the tree
			G_TYPE_INT,		// notebook page index
			G_TYPE_STRING,		// label for the page
			GDK_TYPE_PIXBUF);	// icon for the page

	gtk_tree_view_set_model(GTK_TREE_VIEW(widget), GTK_TREE_MODEL(store));
	column = gtk_tree_view_column_new();

	cell = gtk_cell_renderer_pixbuf_new();
	gtk_tree_view_column_pack_start(column, cell, FALSE);
	gtk_tree_view_column_set_attributes(column, cell, "pixbuf", 0, NULL);

	cell = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(column, cell, TRUE);
	gtk_tree_view_column_set_attributes(column, cell, "text", 1, NULL);

	gtk_tree_view_append_column(GTK_TREE_VIEW(widget), column);

	g_signal_connect (
			G_OBJECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(widget))),
			"changed",
			G_CALLBACK(tree_select_callback),
			notebook);

	//El segundo
	widget2 = glade_xml_get_widget(xml, "tree_keys");
	store2 = gtk_tree_store_new(3,
				G_TYPE_INT,		// key of the keylink
				G_TYPE_STRING,		// shortcout
				G_TYPE_STRING); 	// action
				
	gtk_tree_view_set_model(GTK_TREE_VIEW(widget2), GTK_TREE_MODEL(store2));
	column2 = gtk_tree_view_column_new();
	celda = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(column2,celda, FALSE);
	gtk_tree_view_column_set_attributes(column2, celda, "text", 0, NULL);
	gtk_tree_view_column_set_title(column2, "Action");
	gtk_tree_view_append_column(GTK_TREE_VIEW(widget2), column2);
	
	column3 = gtk_tree_view_column_new();
	celda = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(column3, celda, FALSE);
	gtk_tree_view_column_set_attributes(column3, celda, "text", 1, NULL);
	gtk_tree_view_column_set_title(column3, "Shortcut");
	gtk_tree_view_append_column(GTK_TREE_VIEW(widget2), column3);

	// populating the tree
	while (item->title_tree)
	{
		GtkTreeIter	  iter;
		TreeItem	* child_item = item->child;

		gtk_tree_store_append(store, &iter, NULL);
		gtk_tree_store_set(store, &iter,
				0, gdk_pixbuf_new_from_file(item->pixmap_tree, NULL),
				1, item->title_tree,
				2, gtk_notebook_page_num(GTK_NOTEBOOK(notebook), glade_xml_get_widget(xml, item->widget_name)),
				3, item->title_page,
				4, gdk_pixbuf_new_from_file(item->pixmap_page, NULL),
				-1);

		while (child_item && child_item->title_tree)
		{
			GtkTreeIter	  child_iter;
			
			gtk_tree_store_append(store, &child_iter, &iter);
			gtk_tree_store_set(store, &child_iter,
					0, gdk_pixbuf_new_from_file(child_item->pixmap_tree, NULL),
					1, child_item->title_tree,
					2, gtk_notebook_page_num(GTK_NOTEBOOK(notebook), glade_xml_get_widget(xml, child_item->widget_name)),
					3, child_item->title_page,
					4, gdk_pixbuf_new_from_file(child_item->pixmap_page, NULL),
					-1);
			
			child_item++;
		}

		item++;
	}
	
	// setting up callbacks
	widget = glade_xml_get_widget(xml, "button_close");
	g_signal_connect (G_OBJECT(widget), "clicked", G_CALLBACK(on_preferences_close), dialog);

	return dialog;
}

